#define _GNU_SOURCE                     // только 1-й строкой!
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <math.h>
#include <pthread.h>

// модель с завершаемостью потока
void *RunCounter( void *res ) {         // потоковая функция
   pthread_setcancelstate( PTHREAD_CANCEL_ENABLE, NULL );
   pthread_setcanceltype( PTHREAD_CANCEL_ASYNCHRONOUS, NULL );
   volatile unsigned long *p = (unsigned long*)res;
   while( 1 ) *p += 1; 
   return NULL;
}

int main( int argc, char* argv[] ) {
   int nthr = 2,                        // число потоков
       sec = 1,                         // интервал выполнения
       debug = 0, i;
   if( argc > 1 ) {
      if( atoi( argv[ 1 ] ) != 0 )
         nthr = atoi( argv[ 1 ] );
      else
         printf( "запуск: %s [потоков] [[-]секунд]\n", argv[ 0 ] ),
         exit( EXIT_SUCCESS );
      if( argc > 2 && atoi( argv[ 2 ] ) != 0 ) {
         sec = atoi( argv[ 2 ] );
         if( sec < 0 ) {                // признак отладочной диагностики
            sec = -sec; debug = 1;
         }
      }
   }
   if( debug ) {
      printf( "число потоков %d\n", nthr );
      cpu_set_t mask;
      sched_getaffinity( 0, sizeof( cpu_set_t ), &mask );
      printf( "число доступных процессоров %d\n", CPU_COUNT( &mask ) );
   }
   pthread_t *tid = (pthread_t*)calloc( nthr, sizeof( pthread_t ) );
   unsigned long *res = (unsigned long*)calloc( nthr, sizeof(unsigned long ) );
   for( i = 0; i < nthr; i++ ) 
      if( pthread_create( &tid[ i ], NULL, RunCounter, (void*)( res + i ) ) != 0 )
         printf( "ошибка потока %d: %m", i ), exit( EXIT_FAILURE );
   sleep( sec );
   for( i = 0; i < nthr; i++ ) pthread_cancel( tid[ i ] );
   for( i = 0; i < nthr; i++ ) pthread_join( tid[ i ], NULL );
   free( tid );
   if( debug )
      for( i = 0; i < nthr; i++ ) 
         printf( "%lu%s", res[ i ], ( i == nthr - 1 ? "\n" : " " ) );
   double sum1 = 0, sum2 = 0, sq, md;   // сбор статистики
   for( i = 0; i < nthr; i++ ) {
      md = (double)res[ i ];
      sum1 += md;
      sum2 += md * md;
   }
   md = sum1 / nthr;                    // среднее
   sq = sqrt( sum2 / nthr - md * md );  // СКО
   printf( "операций %.0f : %d * %.0f [+/-%.2f%%]\n",
           sum1, nthr, md, 100. * sq / md );
   free( res );
   return( EXIT_SUCCESS );
}
