#include <unistd.h>
#include <math.h>
#include <iostream> 
#include <iomanip> 
#include <thread>        
#include <string>        
#include <sstream>       

// модель с завершаемостью потока
using namespace std;

void thrfunc( volatile long *res ) {         // потоковая функция
   pthread_setcancelstate( PTHREAD_CANCEL_ENABLE, NULL );
   pthread_setcanceltype( PTHREAD_CANCEL_ASYNCHRONOUS, NULL );
   while( true ) ++*res;
}

class RunCounter : public thread {
private:
   long volatile counter;
public:
   RunCounter() : thread( thrfunc, &counter ), counter( 0 ) {}
   void cancel() {
      ostringstream msg; 
      msg << get_id();
      pthread_cancel( stoul( msg.str() ) );
   }
   long getCount() { return counter; }
};

int main( int argc, char* argv[] ) {
   int nthr = 2,                            // число потоков
       sec = 1;                             // интервал выполнения
   bool debug = false;
   if( argc > 1 ) {
      if( atoi( argv[ 1 ] ) != 0 )
         nthr = atoi( argv[ 1 ] );
      else
         cout << "запуск: " << argv[ 0 ] << " [потоков] [[-]секунд]" << endl,
         exit( EXIT_SUCCESS );
      if( argc > 2 && atoi( argv[ 2 ] ) != 0 ) {
         sec = atoi( argv[ 2 ] );
         if( sec < 0 ) {                // признак отладочной диагностики
            sec = -sec; debug = 1;
         }
      }
   }
   if( debug ) {
      cpu_set_t mask;
      sched_getaffinity( 0, sizeof( cpu_set_t ), &mask );
      cout << "число потоков " << nthr << endl
           << "число доступных процессоров " << CPU_COUNT( &mask ) << endl;
   }
   RunCounter *ar = new RunCounter[ nthr ]; // стартовали потоки
   sleep( sec );
   for( int i = 0; i < nthr; i++ ) ar[ i ].cancel(); 
   for( int i = 0; i < nthr; i++ ) ar[ i ].join(); 
   if( debug )
      for( int i = 0; i < nthr; i++ ) 
         cout << ar[ i ].getCount() << ( i == nthr - 1 ? "\n" : " " );
   double sum1 = 0, sum2 = 0, sq, md;       // сбор статистики
   for( int i = 0; i < nthr; i++ ) {
      md = (double)ar[ i ].getCount();
      sum1 += md;
      sum2 += md * md;
   }
   md = sum1 / nthr;                        // среднее
   sq = sqrt( sum2 / nthr - md * md );      // СКО
   cout << "операций " << (long)sum1 << " : " << nthr << " * "
        << (long)md << " [+/-" << setprecision( 3 )
        << 100. * sq / md << "%]" <<endl; 
   delete [] ar;
   return( EXIT_SUCCESS );
}
