#!/usr/bin/python
# -*- coding: utf-8 -*-
import os
import time
import sys
import signal
import math
import multiprocessing
# параллельные процессы - все OS:

def handler( signum, frame ):                         # обработчик SIGUSR1
    global finish
    finish = True

def RunCounter( arr, num ) :
    global finish
    signal.signal( signal.SIGUSR1, handler )
    counter = 0
    while not finish : counter += 1
    arr[ num ] = counter

if __name__ == '__main__' :                           # для работы в Windows
    multiprocessing.freeze_support()
    nthr = 2                                          # число процессов
    sec = 1                                           # интервал выполнения
    debug = False
    finish = False                                    # флаг завергшения
    try :
        if len( sys.argv ) > 1 :
            nthr = int( sys.argv[ 1 ] )
            if len( sys.argv ) > 2 :
                sec = int( sys.argv[ 2 ] )
                if sec < 0 :                          # признак отладочной диагностики
                    sec = -sec
                    debug = True
    except ValueError :
        print( 'запуск: python[3] {} [потоков] [[-]секунд]'.format( sys.argv[ 0 ] ) )
        sys.exit( 1 )
    if( debug ) :
        print( 'число процессов {}'.format( nthr ) )
        print( 'число доступных процессоров {}'.format( multiprocessing.cpu_count() ) )
    counter = 0
    childs = [] 
    res = multiprocessing.Array( 'i', range( nthr ) ) # массив результатов в shared memory
    for i in range( nthr ) :                          # создание и запуск процессов
        p = multiprocessing.Process( target=RunCounter, args=( res, i ) )
        childs.append( p )
        p.start()
    time.sleep( sec )
    for p in childs : os.kill( p.pid, signal.SIGUSR1 )
    for p in childs : p.join()
    if debug :
        msg = ''
        for i in range( len( res ) ) : msg += '{} '.format( res[ i ] )
        print( '{}'.format( msg ) )
    sum1, sum2, sq, md = 0.0, 0.0, 0.0, 0.0           # сбор статистики
    for i in range( len( res ) ) :
        md = float( res[ i ] )
        sum1 += md
        sum2 += md * md;
    md = sum1 / nthr;                                 # среднее
    sq = math.sqrt( sum2 / nthr - md * md );          # СКО
    print( 'операций {:.0f} : {} * {:.0f} [+/-{:.2f}%]'
           .format( sum1 , nthr, md, 100. * sq / md ) )
    sys.exit( 0 )
